!
!
! Energy corrections for charge defects
! Reciprocal space formalism.
! In the code a Gaussian charge distribution is used
! (extension to other charge models with spherical symmetry is straightforward).
!
! The lattice, self interaction and correction energies are given
! If mode 2 is active the code reads the .cube files (generated by PW&PP),
! makes the differences and calculates the averaged potential along the idir 
! direction. 
!
! Makefile:	A Makefile is provided for compilation
!
! input:
!			i               # mode #1 model charge solution only, #2 read pw files 
!			a11 a12 a13     # cell parameters [Bohr]
!			a21 a22 a23     #
!			a31 a32 a33     #
!			ecut            # ecutoff [Ry]
!			x y z           # charge position [Bohr]
!			q               # charge
!			s               # sigma [Bohr]
!			12.4            # epsilon
!			3               # idir (for potential average <V(idir)>)
!			pot_defQ.cube   # potential, charged def. system	(is read only if mode=2)
!			pot_ref.cube    # potential, reference system		(is read only if mode=2)			 	
!
! Author: 	Giacomo Miceli
!
program correction

	use pwvar	
	
	implicit none
	integer :: i,j,k, npw, nx, ny, nz, idir,&
				numint, ier, mode
	real, parameter :: pi=3.14159265358979323844, hartree2eV=27.211384
	real :: a1(3),a2(3),a3(3),b1(3),b2(3),b3(3) 
	real :: ecut, vol,volg, Gmax, GG,pos(3),q,s,eps, &
			vec(3),Dx,Dy,Dz,r,Elat,self, norm, &
			Eiso,errint
	real, allocatable :: gx(:),gy(:),gz(:), fin(:,:,:)
	double complex, allocatable :: Vg(:,:,:), Vr(:,:,:)
	character (len=80) :: filech,fileref
	
	real :: time0,time1,time2
	
	integer*8 plan
	include 'fftw3.f'
	
!*	input parameters
	read(*,*)mode
	read(*,*)a1
	read(*,*)a2
	read(*,*)a3
	read(*,*)ecut
	read(*,*)pos(1),pos(2),pos(3)
	read(*,*)q
	read(*,*)s
	read(*,*)eps
	read(*,*)idir
	if(mode .eq. 2)then
	read(*,*)filech
	read(*,*)fileref
	endif
	
	call cpu_time(time0)
	
!*	Energy and G^2 are in Ry	
	Gmax=sqrt(ecut)
	call cross_product(a2,a3,vec)
	vol=dot_product(a1,vec)
	npw=nint( 4./3.*pi*Gmax*Gmax*Gmax/( 8*pi*pi*pi /vol ) )
	nx=2*nint( Gmax/( 2*pi/sqrt(dot_product(a1,a1)) ) )
	ny=2*nint( Gmax/( 2*pi/sqrt(dot_product(a2,a2)) ) )
	nz=2*nint( Gmax/( 2*pi/sqrt(dot_product(a3,a3)) ) )
	
!	Real space grid
	Dx=sqrt(dot_product(a1,a1))/nx
	Dy=sqrt(dot_product(a2,a2))/ny
	Dz=sqrt(dot_product(a3,a3))/nz
	
!	Brillouine Zone	
	call cross_product(a2,a3,vec)
	b1=2*pi*vec/vol
	call cross_product(a3,a1,vec)
	b2=2*pi*vec/vol
	call cross_product(a1,a2,vec)
	b3=2*pi*vec/vol
	call cross_product(b2,b3,vec)
	volg=dot_product(b1,vec)
	
	allocate( gx(nx),gy(ny),gz(nz), &
	          Vg(nx,ny,nz), Vr(nx,ny,nz),&
	          fin(nx,ny,nz) )
	          
!*	Reciprocal space mesh 
!	This mesh has been created according to the fftw3 frequencies ordering.
!	In fortran the output of fftw3 is such that k/N = -(N-k+2)/N
	gx(1)=0.0
	do i=2,nx/2+1
		gx(i)=(i-1)*sqrt( dot_product(b1,b1) )
		gx(nx+2-i)=-gx(i)
	enddo
	gy(1)=0.0
	do i=2,ny/2+1
		gy(i)=(i-1)*sqrt( dot_product(b2,b2) )
		gy(ny+2-i)=-gy(i)
	enddo
	gz(1)=0.0
	do i=2,nz/2+1
		gz(i)=(i-1)*sqrt( dot_product(b3,b3) )
		gz(nz+2-i)=-gz(i)
	enddo
	
!	Gaussian charge model in the reciprocal space (excluding the normalization factor)
!	q*exp(-1./2. * s*s * g*g)
!	The average value of the potential in real space has been chosen as follows:
!	V(G=0) = 2*pi/sqrt(vol) * (-q s*s)

!
!   Eiso for a gaussian charge distribution is 1/( 2*sigma*epsilon*sqrt(pi) )
!
	call qag ( f, 0., Gmax, 0.0001, 0.00001, 6, Eiso, errint, numint, ier )
	Eiso=1./pi/eps * Eiso

! 	Lattice energy
	Elat=0.0
	do k=1,nz
		do j=1,ny
			do i=1,nx
				if(i.eq.1 .and. j.eq.1 .and. k.eq.1)cycle
				GG=gx(i)*gx(i) + gy(j)*gy(j) + gz(k)*gz(k)
				if (GG .le. ecut) then
					Elat=Elat+2*pi/eps/vol* ( q * exp(-0.5 * s*s *GG) )**2 / GG
				endif
			enddo
		enddo
	enddo
		
	!Elat=Elat - 2*pi/eps/vol * q*q * s*s

!	Poisson solver and planar averaged potential along direction idir
!
!	Electrostatic potential in reciprocal space
	Vg=0.0
	if (idir.eq.1) then
		do i=2,nx
			GG=gx(i)*gx(i)
			Vg(i,1,1)=4*pi/sqrt(vol)/eps * q * exp(-0.5 * s*s *GG) / GG
		enddo
	elseif (idir.eq.2) then
		do j=2,ny
			GG=gy(j)*gy(j)
			Vg(1,j,1)=4*pi/sqrt(vol)/eps * q * exp(-0.5 * s*s *GG) / GG
		enddo
	else
		do k=2,nz
			GG=gz(k)*gz(k)
			Vg(1,1,k)=4*pi/sqrt(vol)/eps * q * exp(-0.5 * s*s *GG) / GG
		enddo
	endif
			! Fixing the average value of the potential in real space
			!Vg(1,1,1)=2*pi/sqrt(vol)/eps * (-q * s*s)
			Vg(1,1,1)=0.0
			
!	shifting
	if (idir.eq.1) then
		do i=2,nx
			Vg(i,1,1)=Vg(i,1,1)*cexp(cmplx(0.0,-gx(i)*pos(1)))
		enddo
	elseif (idir.eq.2) then
		do j=2,ny
			Vg(1,j,1)=Vg(1,j,1)*cexp(cmplx(0.0,-gy(j)*pos(2)))
		enddo
	else
		do k=2,nz
			Vg(1,1,k)=Vg(1,1,k)*cexp(cmplx(0.0,-gz(k)*pos(3)))
		enddo
	endif	
	
!	Electrostatic potential in real space
	call dfftw_plan_dft_3d(plan,nx,ny,nz,Vg,Vr,FFTW_BACKWARD, FFTW_ESTIMATE)
	call dfftw_execute(plan,Vg,Vr)
	call dfftw_destroy_plan(plan)
	
	Vr=1./sqrt(vol)*Vr

!	Fourier interpolation of the planar averaged electrostatic potential
!	the interpolation points are fixed in the pwvar module (npt)
!	f1(npt) is the result for the model cherge
	fin=real(Vr)
	open(11,file='averagepot.dat',status='unknown')
	if (idir .eq. 1)then
	call interpol(fin(:,1,1),f1,nx,npt)
	write(11,'(A,I2)')'# Planar averaged electrostatic potentials in [V] along direction',idir
	write(11,'(A)')'# for a periodic array of Gaussian charge distributions'
	do i=1,npt
	write(11,*)Dx*nx/npt*(i-1),f1(i)*hartree2eV
	enddo
	else if (idir .eq. 2)then
	call interpol(fin(1,:,1),f1,ny,npt)
	write(11,'(A,I2)')'# Planar averaged electrostatic potentials in [V] along direction',idir
	write(11,'(A)')'# for a periodic array of Gaussian charge distributions'
	do i=1,npt
	write(11,*)Dy*ny/npt*(i-1),f1(i)*hartree2eV
	enddo
	else
	call interpol(fin(1,1,:),f1,nz,npt)
	write(11,'(A,I2)')'# Planar averaged electrostatic potentials in [V] along direction',idir
	write(11,'(A)')'# for a periodic array of Gaussian charge distributions'
	do i=1,npt
	write(11,*)Dz*nz/npt*(i-1),f1(i)*hartree2eV
	enddo
	endif
	
	call cpu_time(time1)
	

!	Difference between potentials of the charged and reference system
!	(cube files from PW)	
	if (mode .eq. 2)then
		open(111,file=filech,status='old')
		open(222,file=fileref,status='old')
		
		call head_cube(111)
		allocate (potch(pwnx,pwny,pwnz))
		call tail_cube(111,potch)
		
		call head_cube(222)
		allocate (potref(pwnx,pwny,pwnz))
		call tail_cube(222,potref)
		
		potch=potref-potch

!	Planar average and Fourier interpolation
!	the interpolation points are fixed in the pwvar module (npt)
!	f2(npt) is the result for < Vpw[q] - Vpw[ref] >
				
		call average(potch,idir)
		write(11,*)
		write(11,*)
		write(11,'(A,I2)')'# Planar averaged electrostatic potentials in [V] along direction',idir
		write(11,'(A)')'# Column 1: Vpw[q] - Vpw[ref]'
		write(11,'(A)')'# Column 2: Vpw[q] - Vpw[ref] - Vgauss[q]'
		
		if (idir .eq. 1)then
		do i=1,npt
		write(11,*)pwdx*pwnx/npt*(i-1),f2(i)*hartree2eV/2.,( f2(i)/2.-f1(i) )*hartree2eV
		enddo
		else if (idir .eq. 2) then
		do i=1,npt
		write(11,*)pwdy*pwny/npt*(i-1),f2(i)*hartree2eV/2.,( f2(i)/2.-f1(i) )*hartree2eV
		enddo
		else
		do i=1,npt
		write(11,*)pwdz*pwnz/npt*(i-1),f2(i)*hartree2eV/2.,( f2(i)/2.-f1(i) )*hartree2eV
		enddo
		endif
		
		call cpu_time(time2)
		
	endif

!REPORT_ 
write(*,*)''
write(*,*)'====================================================================='
write(*,*)' Electrostatic corrections'
write(*,*)'---------------------------------------------------------------------'
write(*,*)''
write(*,*)'INPUT DATA AND SYSTEM INFO'
Write(*,*)
write(*,'(A40)')" Cell parameters [Bohr]:"
write(*,'(40x,3F10.4)')a1
write(*,'(40x,3F10.4)')a2
write(*,'(40x,3F10.4)')a3
write(*,*)''
write(*,'(A40,18X,F12.2)')" Volume cell [Bohr^3]:",vol
write(*,'(A40,18X,F12.2)')" Volume BZ [1/Bohr^3]:",volg
write(*,'(A40)')" Charge position [Bohr]:"
write(*,'(40x,3F10.4)')pos
write(*,'(A40,18X,F12.2)')"Q:",q
write(*,'(A40,18X,F12.2)')"sigma [Bohr]:",s
write(*,'(A40,18X,F12.2)')"epsilon:",eps
write(*,'(A40,18X,F12.2)')" Cutoff Energy [Ry]:",ecut
write(*,'(A40,18X,F12.2)')" Number of plane waves:", real(npw)
write(*,'(A40,18X,3I4)')"FT grid:",nx,ny,nz
write(*,*)''
write(*,*)'!!!! THE AVERAGE ELECTROSTATIC POTENTIAL IS SET TO', Real(Vg(1,1,1)), '!!!!'
write(*,*)''
write(*,*)'REPORT_ CALCULATIONS'
write(*,*)'---------------------------------------------------------------------' 
write(*,*)''
write(*,*)'INTEGRATION'
write(*,*)''
write(*,'(A40,18X,F12.6)')' || I - RESULT || :',errint
write(*,'(A40,18X,I4)')'# integral evaluation:', numint
if (ier .eq. 0)then
write(*,*)'Normal and reliable termination of the routine.'
else
write(*,*)"The integral has not been correctly evaluated:", ier
endif
write(*,*)''
write(*,*)'RESULTS'
write(*,*)''
write(*,'(A40,18X,F12.6)')'Eiso [eV]:',Eiso*hartree2eV
write(*,'(A40,18X,F12.6)')'Epbc [eV]:',Elat*hartree2eV
write(*,'(A40,18X,F12.6)')'Ecorr = (Eiso-Epbc) [eV]:', ( Eiso - Elat )*hartree2eV
write(*,*)
write(*,*)'Planar averaged electrostatic potentials: averagepot.dat'
write(*,*)
if(mode .eq. 1) then
write(*,'(A,F4.1,A)')'The program needed ', time1-time0, ' seconds'
else
write(*,'(A,F4.1,A)')'The program needed ', time2-time0, ' seconds'
endif

contains

! Function |rho|^2 in reciprocal space q and s are read from input
function f ( x )
	real f
	real x
	f=( q*exp(-0.5 * s*s * x*x) )**2
	return
end

end program

!===============================================================================
!===============================================================================

!*******************************************************************************
! AVERAGE
!
! averaged potential along idir direction (no interpolation)
subroutine average(rho,idir)
		
		use pwvar, only: pwnx,pwny,pwnz,pwdx,pwdy,pwdz,f2,npt

		implicit none
		integer :: i,j,k,idir
		real :: summ
		real, dimension(:), allocatable :: inp(:)
		real, dimension (pwnx,pwny,pwnz) :: rho
		
		if (idir .eq. 3) then
		allocate (inp(pwnz))
		do k=1,pwnz
		summ=0.0
		do i=1,pwnx
			do j=1,pwny
				summ=summ+rho(i,j,k)
			enddo
		enddo
		inp(k)=summ/pwnx/pwny
		enddo
		call interpol(inp,f2,pwnz,npt)
		
		else if (idir .eq. 2) then
		allocate (inp(pwny))
		do j=1,pwny
		summ=0.0
		do i=1,pwnx
			do k=1,pwnz
				summ=summ+rho(i,j,k)
			enddo
		enddo
		inp(j)=summ/pwnx/pwnz
		enddo
		call interpol(inp,f2,pwny,npt)
		
		else
		allocate (inp(pwnx))
		do i=1,pwnx
		summ=0.0
		do j=1,pwny
			do k=1,pwnz
				summ=summ+rho(i,j,k)
			enddo
		enddo
		inp(i)=summ/pwny/pwnz
		enddo
		call interpol(inp,f2,pwnx,npt)
		
		endif

end subroutine

!*******************************************************************************
!  CROSS_PRODUCT
!
!  Returns the right-handed vector cross product of two 3-vectors:  C = A x B.
!*******************************************************************************

      SUBROUTINE CROSS_PRODUCT (A, B, C)                                         
      IMPLICIT NONE
      REAL, DIMENSION(3), INTENT (IN)    :: A
      REAL, DIMENSION(3), INTENT (IN)    :: B 
      REAL, DIMENSION(3), INTENT (OUT)   :: C

      C(1) = A(2)*B(3) - A(3)*B(2)
      C(2) = A(3)*B(1) - A(1)*B(3)
      C(3) = A(1)*B(2) - A(2)*B(1)

      RETURN

      END SUBROUTINE CROSS_PRODUCT
